<?php

require_once 'exercises/exercise.php';

class cloze_exercise extends Exercise
{

    // constructor
    function cloze_exercise($xml = null)
    {
        parent::Exercise($xml);

        $this->type = 'cloze_exercise';

        if (!$xml) {  // construct exercise using request parameters
            $this->question        = $this->questionFromInput();
            $this->answerArray     = $this->getAnswerArray();
            $this->correctArray    = $this->getCorrectArray();
            $this->numberOfAnswers = $this->getNumberOfAnswers();
        }
        else {  // construct exercise using XML
            $this->question        = $this->questionFromXML($xml);
            $this->answerArray     = $this->getAnswerArray();
            $this->correctArray    = $this->getCorrectArray();
            $this->numberOfAnswers = $this->getNumberOfAnswers();
        }
    }





    ###########################################
    #                                         #
    #   " p u b l i c "   f u n c t i o n s   #
    #                                         #
    ###########################################



    /**
     * Returns the exercise question.
     *
     * @param view In a cloze exercise, the question looks different depending
     *             on the action which is done. Possible values for $view are
     *             'edit' and 'show', the default is 'show'.
     */
    function getQuestion($view = 'show')
    {
        if ($view == 'edit') {
            return $this->getQuestionEdit();
        }
        else {
            return $this->getQuestionShow();
        }
    }



    /**
     * Get the exercise in a format that enables to store it in the database (i.e. XML format).
     */
    function getExerciseConstruct()
    {
        $xml = '<TestItem ID="'.$this->type.'" Type="Cloze" Language="'.$this->language.'">'
            .'<Name>'.htmlspecialchars($this->name).'</Name>'
            .$this->questionToXML()
            .'<Hint>'.htmlspecialchars($this->hint).'</Hint>'
            .'<CommentField Visible="'.$this->commentAllowed.'"/>'
            .'</TestItem>';

        return vips_filter_xml_string($xml);
    }



    /**
     * Returns the exercise in im-/export format (plain text).
     */
    function exportText ()
    {
        // $result = parent::exportText("L'text");

        $result = 'Name: '.$this->name."\n";
        $result .= "L'text: ".$this->getQuestionEdit()."\n";
        $result .= "\\L'text\n";

        if ($this->hint != '')
        {
            $result .= "Tipp:\n";
            $result .= $this->hint."\n";
            $result .= "\\Tipp\n";
        }

        return $result;
    }



    /**
     * Creates a template for editing a cloze exercise. NOTE: As a cloze
     * exercise has no special fields (it consists only of the question), an
     * empty template will be returned.
     *
     * @returns The (empty!) template for editing this exercise
     */
    function getEditTemplate()
    {
        global $vipsTemplateFactory;

        $template = $vipsTemplateFactory->open('exercises/edit_cloze_exercise');

        return $template;
    }



    /**
     * Creates a template for solving a cloze_exercise.
     *
     * @returns The template
     */
    function getSolveTemplate($solution_xml = null)
    {
        global $vipsTemplateFactory;

        if (isset($solution_xml)) {
            $student_array = $this->getTagsFromXML($solution_xml, 'answer');
        }

        $template = $vipsTemplateFactory->open('exercises/solve_cloze_exercise');
        $template->set_attributes(array(
            'exercise_hint'     => $this->hint,
            'exercise_question' => $this->question,
            'student_array'     => $student_array));

        return $template;
    }



    /**
     * Fills a WIKI template with all necessary values. If, additionally, a
     * solution is processed to it, it will also be set in.
     *
     * @param $solution (optional) the solution XML
     * @return the filled WIKI template object
     */
    function getWikiTemplate($solutionXML = null)
    {
        global $vipsTemplateFactory;

        // handle solution
        if (isset($solutionXML)) {
            $student_array = $this->getTagsFromXML($solutionXML, 'answer');
        }

        $template = $vipsTemplateFactory->open('exercises/wiki_cloze_exercise');
        $template->set_attributes(array(
            'name'          => $this->name,
            'question'      => $this->getQuestionShow(),
            'hint'          => $this->hint,
            'student_array' => $student_array
            ));

        return $template;
    }



    function getCorrectedWikiTemplate($solution_xml)
    {
        global $vipsTemplateFactory;

        $student_array = $this->getTagsFromXML($solution_xml, 'answer');

        $template = $vipsTemplateFactory->open('exercises/wiki_cloze_exercise_corrected');
        $template->set_attributes(array(
            'name'          => $this->name,
            'hint'          => $this->hint,
            'question'      => $this->getQuestionShow(),
            'correct_array' => $this->correctArray,
            'student_array' => $student_array
            ));

        return $template;
    }



    /**
     * Returns the number of blanks in the cloze text.
     */
    function getNumberOfAnswers()
    {
        return count($this->getCorrectArray());
    }



    /**
     * Returns the text passages between the gaps. Not really "answers", but a counterpart.
     */
    function getAnswerArray()
    {
        if (!$this->question) {
            return false;
        }

        $answer_array = array();

        foreach ($this->question as $elem) {
            if (!is_array($elem)) {
                $elem = str_replace(array('\\[', '\\]'), array('[', ']'), $elem);
                $answer_array[] = $elem;
            }
        }

        return $answer_array;
    }



    /**
     * Returns all the correct solutions in an array.
     */
    function getCorrectArray()
    {
        if (!$this->question) {
            return false;
        }

        $correct_array = array();

        foreach ($this->question as $elem) {
            if (is_array($elem)) {
                $correct_options = array();
                foreach ($elem as $option) {
                    if ($option['points'] == 1) {  // answer counts as correct if it gives 1 point
                        $correct_options[] = $option['content'];
                    }
                }
                $correct_array[] = $correct_options;
            }
        }
        return $correct_array;
    }

    /**
     * Returns all the partly correct solutions in an array.
     */
    function getPartlyCorrectArray()
    {
        if (!$this->question) {
            return false;
        }

        $correct_array = array();

        foreach ($this->question as $elem) {
            if (is_array($elem)) {
                $correct_options = array();
                foreach ($elem as $option) {
                    if ($option['points'] == 0.5) {  // partly correct answer
                        $correct_options[] = $option['content'];
                    }
                }
                $correct_array[] = $correct_options;
            }
        }
        return $correct_array;
    }

    /**
     * Returns all variables which are peculiar to this exercise type.
     *
     * @returns array with the key 'partly_correct_array'
     */
    function getQuirks()
    {
        return array('partly_correct_array' => $this->getPartlyCorrectArray());
    }



    /**
     * Generates an XML-String containing the solution of the student.
     *
     * Gets its values to be stored in the XML-String from the
     * Get/Post environment.
     *
     * @param exercise_id the id of the exercise the solution related to.
     * @param p_number_of_answers (deprecated) the number of answers possible (This is a class variable!)
     * @return XML the XML-String containing the solutions of the student.
     *
     * @todo names of the exml entities are not well choosen. A change will have
     * impact on other parts of the code, of course.
     */
    function genSolution($exercise_id, $number_of_answers = null)
    {
        $noa = isset($number_of_answers) ? $number_of_answers : $this->numberOfAnswers;
        $xml = '<solution id="'.$exercise_id.'">';
        for ($i = 0; $i < $noa; $i++)
        {
            $xml .= '<answer id="'.$i.'">'
                .htmlspecialchars(vips_get_string_param('answer_'.$i, true))
                .'</answer>';
        }
        $xml .= '</solution>';

        return vips_filter_xml_string($xml);
    }



    /**
     * Evaluates a student's solution.
     *
     * @param exercise Unused! :-)
     * @param solution The solution XML string as returned by genSolution().
     */
    function evaluate($exercise, $solution)
    {
        $points = 0;
        $safe   = true;

        $student_answers = $this->getTagsFromXML($solution, 'answer');

        $blank = 0;
        foreach ($this->question as $elem) {
            if (is_array($elem)) {  // blank
                $match = false;
                foreach ($elem as $option) {  // different answer options
                    if ($student_answers[$blank] == $option['content']) {
                        $points += $option['points'];  // positive or negative points
                        $match = true;  // student's answer matches one of the lecturer's defaults
                    }
                }
                // If the student didn't give one of the answers the lecturer provided, consider the exercise as corrected unsafe!
                if (!$match && $student_answers[$blank] != '') {
                    $safe = false;
                }
                $blank++;
            }
        }

        $percent = $points * 100 / $this->numberOfAnswers;

        // $safe ist nur dann wahr, wenn der Student ausschlielich die Antworten gegeben hat, die der Dozent fr die Lcken vorgesehen hat (gleich, ob mit positiven oder negativen Punkten belegt). Sobald eine Antwort in einer Lcke von den Vorgaben abweicht, kann die Aufgabe nicht mehr als sicher korrigiert angesehen werden.
        return array(
            1         => $percent,
            2         => $safe,
            'percent' => $percent,
            'safe'    => $safe);
    }





    #######################################
    #                                     #
    #   h e l p e r   f u n c t i o n s   #
    #                                     #
    #######################################



    /**
     * Returns the exercise for the lecturer. Clozes are represented by square brackets. Special characters are escaped.
     */
    function getQuestionEdit()
    {
        $text = '';
        foreach ($this->question as $elem) {
            if (!is_array($elem)) {  // no blank
                // $text .= addcslashes($elem, '[]');  // escape square brackets
                $text .= $elem;
            }
            else {  // blank
                $text .= '[[';
                for ($i = 0; $i < count($elem); $i++) {
                    if ($i) {
                        $text .= '|'; // '|' between options
                    }
                    if ($elem[$i]['points'] == 0) {
                        $text .= '-';
                    } else if ($elem[$i]['points'] == 0.5) {
                        $text .= '~';
                    }
                    // $text .= addcslashes($elem[$i]['content'], '[]|');  // escape square bracktes and pipe symbol
                    $text .= $elem[$i]['content'];
                }
                $text .= ']]';
            }
        }
        return $text;
    }



    /**
     * Returns the exercise for the student. Each cloze is represented by an array. Special characters are not escaped.
     */
    function getQuestionShow()
    {
        $question = array();

        foreach ($this->question as $elem) {
            if (!is_array($elem)) {  // no blank
                // $elem = str_replace(array('\\[', '\\]'), array('[', ']'), $elem);  // unescape square brackets
                $question[] = $elem;
            }
            else {  // blank
                $question[] = array();  // Das Array ist leer. Wenn in Zukunft der Aufgabentyp noch erweitert wird, kann da aber auch was drinstehen (z.B. Greninformationen).
            }
        }
        return $question;
    }



    /**
     * Converts plain text ("foo bar [blank] text...") to array.
     */
    function questionFromInput()
    {
        $question = vips_get_string_param('exercise_question', true);
        // Mehrfachklammern [[ [[[ [[[[ ]] ... (OHNE \ davor!) werden zu einfachen zusammengefat
        // $question = preg_replace(array('/(?<!\\\\)\[+/', '/(?<!\\\\)\]+/'), array('[', ']'), $question);

        /* eine [ OHNE \ davor (lookbehind assertion), gefolgt von mglichst wenig (ungreedy) bel. Zeichen, gefolgt von einer ] OHNE \ davor (lookbehind assertion); das alles als "split delimiter" rund eingeklammert und mit ins Array gepackt */
        // $question_array = preg_split('/((?<!\\\\)\[.*?(?<!\\\\)\])/s', $question, -1, PREG_SPLIT_DELIM_CAPTURE);
        $question_array = preg_split('/(\[\[.*?\]\])/s', $question, -1, PREG_SPLIT_DELIM_CAPTURE);

        // Assertion:
        // $question_array contains text elements (maybe with \[ and \] in it) and blanks (surrounded by [ and ] ).

        for ($i = 0; $i < count($question_array); $i++) {
            if (!preg_match('/^\[\[(.*)\]\]$/s', $question_array[$i], $matches)) {  // no blank
                // change '\[' to '[' and '\]' to ']'
                // $question_array[$i] = str_replace(array('\\[', '\\]'), array('[', ']'), $question_array[$i]);
            }
            else {  // blank
                // remove outer square brackets and rare stuff (\n, \r) from the blank
                // $question_array[$i] = trim($question_array[$i], '[]');
                $question_array[$i] = $matches[1];
                $question_array[$i] = str_replace(array("\n", "\r"), '', $question_array[$i]);

                // $answers = preg_split('/(?<!\\\\)\|/', $question_array[$i]);  // delimiter: '|', NOT '\|'!
                $answers = explode('|', $question_array[$i]);  // delimiter: '|', NOT '\|'!
                $question_array[$i] = array();
                foreach ($answers as $answer) {
                    $points = 1;
                    // $answer = str_replace(array('\\|', '\\[', '\\]'), array('|', '[', ']'), $answer);

                    if ($answer != '') {
                        if ($answer[0] == '-') {
                            $points = 0;
                            $answer = substr($answer, 1);
                        } else if ($answer[0] == '~') {
                            $points = 0.5;
                            $answer = substr($answer, 1);
                        }
                    }

                    $question_array[$i][] = array('content' => $answer, 'points' => $points);
                }
            }
        }

        return $question_array;
    }



    /**
     * Uses an XML string to create this exercise's question (object parameter).
     * All the special characters are stored <b>unescaped</b> in the exercise
     * question.
     *
     * @param xml The exercise's XML representation (as returned by function
     *            questionToXML()).
     */
    function questionFromXML($xml)
    {
        // with 's' modifier, '.' matches all characters INCLUDING newline!
        preg_match('/<Question>(.*)<\/Question>/s', $xml, $question);
        $question = $question[1];  // without the Question tags

        // ungreedy match all
        $question = preg_split('/<Cloze>(.*?)<\/Cloze>/s', $question, -1, PREG_SPLIT_DELIM_CAPTURE);

        // first element in $question is ALWAYS text (maybe empty string if question begins with a blank)

        for ($i = 0; $i < count($question); $i++) {
            if (!preg_match('/<Answer Points=".*">.*<\/Answer>/s', $question[$i])) {  // no blank
                $question[$i] = html_entity_decode($question[$i]);
            }
            else {  // blank
                // splitte an einem "<Answer ", das NICHT am Zeilenanfang steht
                $answers = preg_split('/(?!^)(?=<Answer )/', $question[$i]);
                $question[$i] = array();
                foreach ($answers as $answer) {
                    $content = array_shift($this->getTagsFromXML($answer, 'Answer'));
                    $points  = array_shift($this->getAttributesFromXML($answer, 'Answer', 'Points'));

                    $question[$i][] = array('content' => $content, 'points' => $points);
                }
            }
        }
        return $question;
    }



    /**
     * Converts this object's question to XML. The special characters (square brackets, pipe symbol, backslash) are
     * stored unescaped.
     */
    function questionToXML()
    {
        $xml = '<Question>';
        foreach ($this->question as $elem) {
            if (!is_array($elem)) {  // plain text => no blank
                $xml .= htmlspecialchars($elem);
            }
            else {  // blank
                $xml .= '<Cloze>';
                foreach ($elem as $answer) {
                    $content = htmlspecialchars($answer['content']);
                    $points  = $answer['points'];

                    $xml .= '<Answer Points="'.$points.'">'.$content.'</Answer>';
                }
                $xml .= '</Cloze>';
            }
        }
        $xml .= '</Question>';
        return $xml;
    }

}  // end class

?>
