<?php
/*
 * oc_exercise.php - Vips plugin for Stud.IP
 * Copyright (c) 2003-2005  Erik Schmitt, Philipp Hgelmeyer
 * Copyright (c) 2005-2006  Christa Deiwiks
 * Copyright (c) 2006-2007  Elmar Ludwig, Martin Schrder
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 */

require_once 'exercises/exercise.php';

/**
 * XXX detailed description
 *
 * @author    Philipp Hgelmeyer
 * @copyright XXX
 * @abstract
 */
class oc_exercise extends Exercise
{
    function oc_exercise($xml = null)
    {
        parent::Exercise($xml);

        $this->type = 'oc_exercise';

        if (!isset($xml)) {  // construct exercise using request parameters
            $this->answerArray = array(vips_get_string_param('answer_0', true));
        }

        else {  // construct exercise using xml
            $this->answerArray     = $this->getTagsFromXML($xml, 'Answer');
            $this->numberOfAnswers = 1;
        }
    }

    /**************************************************************************/
    /*
    /* Evaluates the solution
    /*
    /**************************************************************************/

    /**
     *    Evaluates a solution.
     *
     *    @abstract
     *    @access public
     *    @returns int (from 0 to 100 ) as an % value
     */

    function evaluate($exercise,$solution) {
        // get the xml representation
        global $auth;
        $m_shell_output = "";
        $punkte = 0;
        if (!$dom_aufgabe = domxml_open_mem(utf8_encode($exercise)))
        {
            echo "Error while parsing the document\n";
            exit;
        }
        if (!$dom_loesung = domxml_open_mem(utf8_encode($solution)))
        {
            echo "Error while parsing the document\n";
            exit;
        }
        $m_return[1] = 0; // score
        $m_return[2] = 0; // safe
        $m_return['percent'] = 0;
        $m_return['safe']    = 0;

        return $m_return;
    }



    function downloadSolution($p_exercise_name) {
        $p_solution = vips_get_unquoted_string_param("answer_0", true);
        $download_link = '';
        $tmpfname = tempnam("vips/temp", $p_exercise_name);

        $handle = fopen($tmpfname, "w");
        fwrite($handle, $p_solution);
        fclose($handle);

        if (!copy($tmpfname, 'vips/download/'.$p_exercise_name.'.txt')) {  // copy tmpfile to a directory, where it can be downloaded.
            print ("failed to copy $tmpfname...<br>\n");
        }
        else {
            $download_link = "<a href=\"vips/download/$p_exercise_name.txt\">$p_exercise_name.txt</a>";
        }
        unlink($tmpfname);  //delete file from temp folder
        return $download_link;
    }



    /**
     * Uploads a file as solution.
     *
     * @returns The file content
     */
    function uploadSolution()
    {
        $solution = null;

        if (!empty($_FILES['pl_userfile'])) {
            if (is_uploaded_file($_FILES['pl_userfile']['tmp_name'])) {
                $solution = file_get_contents($_FILES['pl_userfile']['tmp_name']);
            }
        }
        else {
            echo _('Fehler beim Hochladen!');
        }

        return $solution;
    }



    /**************************************************************************/
    /* Queries the query inserted by the student with the corresponding code of
    /* the exercise.
    /* @param int exercise_id the id of the exercise that sould be queried.
    /* @param string exerciseXML the exercise code containing the info about the
    /*                           exercise
    /**************************************************************************/

    function evaluateQuery($p_user_id, $p_number = 1) {

        $username = get_solver_name($p_user_id);

        $answer = vips_get_unquoted_string_param("answer_0", true);
        $query  = vips_get_unquoted_string_param("pl_query", true);

        $shell_output = "";

        $fp = fsockopen("mvc.ikw.uni-osnabrueck.de", 2212); //, $errno, $errstr, 30);
        if (!$fp) {
            echo "ERROR fsockopen: PROLOG compiler couldn't be reached'."; //"$errstr ($errno)<br />\n";
        } else {
            $queryXML = "<octave>\n<user>\n$username\n</user>\n<number>\n$p_number\n</number>\n<type>\n$query\n</type>\n<exercise>\n".$answer."\n</exercise>\n</octave>\n";

            //echo "<br>pl_exercise.php evaluateQuery() <pre>".$queryXML."</pre>";
            fputs($fp, $queryXML);
            while (!feof($fp)) {
                $shell_output .= fgets($fp);
            }
            echo "<br>shell_output: $shell_output ";
            fclose($fp);
        }

        list($line1, $line2, $line3, $rest) = split("%", $shell_output);
        list($rest2,$info)                  = split("\n", $rest, 2);
        return $info;
    }



    /**************************************************************************/
    /*
    /* XML-generating routines
    /*
    /**************************************************************************/



    /**
     * Generates generates an XML-String containing the solution of the
     * student.
     *
     * Gets ist values to be stored in the XML-String from the
     * Get/Post environment.
     *
     * @param int $exercise_id the id of the exercise the solution related to.
     * @param int $p_number_of_answers the number of answers possible (??? not a class variable, ES)
     * @return XML the XML-String containing the solutions of the student.
     *
     * @todo names of the exml entities are not well choosen. A change will have
     * impact on other parts of the code, of course.
     */

    function genSolution($p_exercise_id, $p_number_of_answers = null) {

        $i              = 0;
        $msg    = "<solution id=\"".$p_exercise_id."\">";

        $msg .= "<answer id=\"0\">";
        $msg .= "<body>".htmlspecialchars(vips_get_string_param("answer_0", true))."</body>";
        $msg .= "</answer>";

        $msg.= "</solution>";
        return vips_filter_xml_string($msg);
    }



    /**
     * Generate the description of an exercise in an XML-String.
     *
     * Gets its values to be stored in the XML-String from the
     * Get/Post environment.
     *
     * @return XML the XML-String describing the exercise in terms of its
     * name, questions, wrong and correct solutions etc.
     */

    function getExerciseConstruct()
    {
        $xmlop  = '<TestItem ID="'.$this->type.'" Type="Octave" Language="'.$this->language.'">';
        $xmlop .= "<Name>".htmlspecialchars($this->name)."</Name>";
        // In this special Case the hint is displayed inside a textbox to provide comments
        // and code fragments to the student.
        $xmlop .= "<Hint>".htmlspecialchars($this->hint)."</Hint>";
        $xmlop .= '<CommentField Visible="'.$this->commentAllowed.'"></CommentField>';
        $xmlop .= "<Question>";
        $xmlop .= "<Paragraph>".htmlspecialchars($this->question)."</Paragraph>";
        $xmlop .= "</Question>";
        $xmlop .= "<Answer>".htmlspecialchars($this->answerArray[0])."</Answer>";
        $xmlop .= "</TestItem>";

        return vips_filter_xml_string($xmlop);
    }

    /**
     * Export this exercise to plain text format.
     */
    function exportText ()
    {
        $result = parent::exportText('OC-Frage');

        $answer_text = $this->answerArray[0];

        if ($answer_text != '')
        {
            $result .= "Antwort:\n";
            $result .= $answer_text."\n";
            $result .= "\\Antwort\n";
        }

        return $result;
    }



    /**
     * Creates a template for editing an oc_exercise.
     *
     * @returns The template for editing this exercise
     */
    function getEditTemplate()
    {
        global $vipsTemplateFactory;

        $template = $vipsTemplateFactory->open('exercises/edit_oc_exercise');
        $template->set_attributes(array(
            'answer_array' => $this->getAnswerArray()));

        return $template;
    }



    /**
     * Creates a template for solving an oc_exercise.
     *
     * @returns The template
     */
    function getSolveTemplate($solution_xml = null)
    {
        global $vipsTemplateFactory;

        if (vips_get_submit_param('pl_exercise_mode_upload')) {
            $student_array = array($this->uploadSolution());
        }
        else if (isset($solution_xml)) {
            $student_array = $this->getTagsFromXML($solution_xml, 'answer');
        }

        $template = $vipsTemplateFactory->open('exercises/solve_oc_exercise');
        $template->set_attributes(array(
            'exercise_question' => $this->question,
            'exercise_hint'     => $this->hint,
//             'answer_array'      => $this->answerArray,
            'student_array'     => $student_array,
            'pl_query'          => vips_get_unquoted_string_param('pl_query', true),
            'pl_count'          => max(1, vips_get_int_param('pl_count')),
            'pl_out'            => vips_get_string_param('pl_out')));
//             'label_yes'         => $this->labelYes,
//             'label_no'          => $this->labelNo,
//             'evaluation_mode'   => $this->evaluationMode));

        return $template;
    }

}
?>
