<?php
/*
 * sc_exercise.php - Vips plugin for Stud.IP
 * Copyright (c) 2003-2005  Erik Schmitt, Philipp Hgelmeyer
 * Copyright (c) 2005-2006  Christa Deiwiks
 * Copyright (c) 2006-2008  Elmar Ludwig, Martin Schrder
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 */

require_once 'exercises/exercise.php';

/**
 * XXX detailed description
 *
 * @author    XXX
 * @copyright XXX
 * @abstract
 * @TODO Fix dozent view.
 */
class sc_exercise extends Exercise
{
    var $evaluationMode,
        $mistakeComment;

    // constructor
    function sc_exercise($xml = null)
    {
        parent::Exercise($xml);

        $this->type = 'sc_exercise';

        if (!isset($xml)) {  // construct exercise using request parameters
            $this->answerArray     = $this->answerArrayFromInput();
            $this->correctArray    = $this->correctArrayFromInput();
            $this->numberOfAnswers = $this->computeNumberOfAnswers();
            $this->mistakeComment  = vips_get_string_param('mistake_comment', true);
        }
        else {  // construct exercise using XML
            $dom_xml = $this->getDomXML($xml);
            $answers = $dom_xml->get_elements_by_tagname('Answer');

            foreach ($answers as $node)
            {
                $group = $node->get_attribute('group');
                if ($group == '')
                    $group = 0;

                $this->answerArray[$group][] = utf8_decode($node->get_content());

                if ($node->get_attribute('Correct') == '1')
                    $this->correctArray[$group] = count($this->answerArray[$group]) - 1;
            }

            $this->numberOfAnswers = $this->computeNumberOfAnswers();
            $this->mistakeComment  = reset($this->getTagsFromXML($xml, 'MistakeComment'));
        }
    }



    /**
     * Counts the exercise's number of answers.
     *
     * @returns The number of answers in the exercise
     */
    function getNumberOfAnswers()
    {
        return implode(',', $this->numberOfAnswers);
    }



    /**
     * Computes the default maximum points which can be reached in this
     * exercise, dependent on the number of groups.
     *
     * @returns The maximum points
     */
    function getMaxPoints()
    {
        return count($this->numberOfAnswers);
    }



    /**
     * Returns all variables which are peculiar for this exercise type.
     *
     * @returns An array with the keys 'evaluation_mode', 'mistake_comment' and
     *          'change_input_field_size'
     */
    function getQuirks()
    {
        for ($group = 0; $group < count($this->numberOfAnswers); $group++) {
            for ($i = 0; $i < $this->numberOfAnswers[$group]; $i++) {
                if (vips_get_submit_param('increase_input_field_size_'.$group.'_'.$i)) {
                    $change_input_field_size = array(
                        'group'       => $group,
                        'alternative' => $i,
                        'size'        => 'large');
                }
                else if (vips_get_submit_param('decrease_input_field_size_'.$group.'_'.$i)) {
                    $change_input_field_size = array(
                        'group'       => $group,
                        'alternative' => $i,
                        'size'        => 'small');
                }
            }
        }

        return array(
            'evaluation_mode'         => $this->evaluationMode,
            'mistake_comment'         => $this->mistakeComment,
            'change_input_field_size' => $change_input_field_size);
    }



    function setDefaultValues()
    {
        $this->numberOfAnswers = array(5);
    }




    /**************************************************************************/
    /*
    /* Evaluates the solution
    /*
    /**************************************************************************/

    /**
     *    Evaluates a solution.
     *
     *    @abstract
     *    @access public
     *    @returns int (from 0 to 100 ) as an % value
     */
    function evaluate($exercise, $solution)
    {
        // get the xml representation
        $dom_xml = $this->getDomXML($exercise);
        $answers = $dom_xml->get_elements_by_tagname("Answer");
        $mc_auswertung = $this->getEvaluationMode();
        $punkte = 0;

        $answer_array = array();
        $correct_array = array();

        foreach ($answers as $node)
        {
            $group = $node->get_attribute("group");
            if ($group == '') $group = 0;

            $answer_array[$group][] = html_ready(utf8_decode($node->get_content()));
            if ($node->get_attribute("Correct") == '1')
                $correct_array[$group] = count($answer_array[$group]) - 1;
        }

        $student_array = $this->getTagsFromXML($solution, "answer");

        for ($group = 0; $group < count($correct_array); ++$group) {
            if ($student_array[$group] == $correct_array[$group])
                ++$punkte;
            else if ($mc_auswertung == 1 || $mc_auswertung == 2)
                --$punkte;      // abzug bei falscher antwort
        }

        if ($punkte < 0 && $mc_auswertung != 2)
            $punkte = 0;

        $prozent = $punkte / count($correct_array) * 100;
        $safe = 1;

        $return[1]         = $prozent;
        $return[2]         = $safe;
        $return[5]         = $punkte;
        $return['percent'] = $prozent;
        $return['safe']    = $safe;
        return $return;
    }



    /**
     * Generates an XML-String containing the solution of the student.
     *
     * Gets its values to be stored in the XML-String from the
     * Get/Post environment.
     *
     * @param int $exercise_id the id of the exercise the solution related to.
     * @param int $p_number_of_answers (deprecated) the number of answers possible (This is a class variable!)
     * @return XML the XML-String containing the solutions of the student.
     *
     * @todo names of the exml entities are not well choosen. A change will have
     * impact on other parts of the code, of course.
     */

    function genSolution($exercise_id, $p_number_of_answers = null)
    {
        // NOTE number_of_answers contains comma separated values here
        $num_answers = isset($p_number_of_answers) ? explode(',', $p_number_of_answers) : $this->numberOfAnswers;
        $xml = '<solution id="'.$exercise_id.'">';

        for ($group = 0; $group < count($num_answers); $group++) {
            $xml .= '<answer id="'.$group.'">';
            $xml .= '<body>'.htmlspecialchars(vips_get_int_param('answer_'.$group)).'</body>';
            $xml .= '</answer>';
        }

        $xml .= '</solution>';
        return vips_filter_xml_string($xml);
    }



    /**
     * Generate the description of an exercise in an XML-String.
     *
     * Gets its values to be stored in the XML-String from the
     * Get/Post environment.
     *
     * @return XML the XML-String describing the exercise in terms of its
     * name, questions, wrong and correct solutions etc.
     */

    function getExerciseConstruct()
    {
        $xmlop  = "<TestItem ID=\"" . $this->type . "\" Type=\"SingleChoice\" Language=\"" . $this->language . "\">";
        $xmlop .= "<Name>".htmlspecialchars($this->name)."</Name>";
        $xmlop .= "<Hint>".htmlspecialchars($this->hint)."</Hint>";
        $xmlop .= "<CommentField Visible=\"".$this->commentAllowed."\"></CommentField>";
        $xmlop .= "<Question>";
        $xmlop .= "<Paragraph>".htmlspecialchars($this->question)."</Paragraph>";
        $xmlop .= "</Question>";
        $xmlop .= "<EvalMode>".htmlspecialchars($this->evaluationMode)."</EvalMode>";
        $xmlop .= '<MistakeComment>'.htmlspecialchars($this->mistakeComment).'</MistakeComment>';

        $group = 0;
        while (count($this->answerArray[$group])) {
            for ($i = 0; $i < count($this->answerArray[$group]); ++$i) {
                $correct = $i == $this->correctArray[$group] ? '1' : '0';  // 1 indicates correct
                $xmlop .= '<Answer group="'.$group.'" Correct="'.$correct.'">'.htmlspecialchars($this->answerArray[$group][$i]).'</Answer>';
            }
            ++$group;
        }

        $xmlop .= "</TestItem>";

        return vips_filter_xml_string($xmlop);
    }

    /**
     * Export this exercise to plain text format.
     */
    function exportText ()
    {
        $result = parent::exportText('SC-Frage');

        for ($group = 0; $group < count($this->answerArray); ++$group)
        {
            if ($group > 0)
                $result .= "--\n";

            for ($i = 0; $i < count($this->answerArray[$group]); ++$i)
            {
                if ($this->correctArray[$group] == $i)
                    $result .= '+';

                $result .= 'Antwort: '.$this->answerArray[$group][$i]."\n";
            }
        }

        return $result;
    }



    /**
     * Creates a template for editing a sc_exercise.
     *
     * @returns The template for editing this exercise
     */
    function getEditTemplate()
    {
        global $vipsTemplateFactory, $g_outlet, $vipsPlugin;

        // NOTE number_of_answers contains comma separated values here
        $number_of_answers = explode(',', vips_get_string_param('number_of_answers'));

        if ($number_of_answers[0] == '') {
            $number_of_answers = array();
        }

        $answer_array = array();

        for ($group = 0; $group < count($number_of_answers); ++$group) {
            for ($alternative = 0; $alternative < $number_of_answers[$group]; ++$alternative) {
                $answer_array[$group][] =
                    vips_get_string_param('answer_'.$group.'_'.$alternative, true);
            }
        }

        $correct_array = array();

        for ($group = 0; $group < count($number_of_answers); ++$group) {
            $correct_array[] = vips_get_int_param('correct_'.$group);
        }

        for ($i = 0; $i < count($number_of_answers); ++$i) {
            if (vips_get_submit_param('add_answer_'.$i)) {  // add new answer field
                $add_answer = $i;
            }
        }

        if (isset($add_answer)) {
            $number_of_answers[$add_answer] += 1;           // add new answer field
        } else if (vips_get_submit_param('add_group')) {    // add new group
            $number_of_answers[] = 3;                       // with three empty answers
        } else {
            $answer_array = $this->getAnswerArray();
            $correct_array = $this->getCorrectArray();
            $number_of_answers = $this->numberOfAnswers;
        }

        $template = $vipsTemplateFactory->open('exercises/edit_sc_exercise');
        $template->set_attributes(array(
            'exercise_id'             => vips_get_int_param('exercise_id'),
            'answer_array'            => $answer_array,
            'correct_array'           => $correct_array,
            'number_of_answers'       => implode(',', $number_of_answers),
            'evaluation_mode'         => $this->evaluationMode,
            'change_input_field_size' => $this->getQuirk('change_input_field_size'),
            'picturepath'             => $g_outlet->picturepath,
            'vips_imagepath'          => $vipsPlugin->getPluginPath().'/images/'));

        return $template;
    }



    /**
     * Creates a template for solving an sc_exercise.
     *
     * @returns The template
     */
    function getSolveTemplate($solution_xml = null)
    {
        global $vipsTemplateFactory;

        if (isset($solution_xml)) {
            $student_array = $this->getTagsFromXML($solution_xml, 'answer');
        }

        $template = $vipsTemplateFactory->open('exercises/solve_sc_exercise');
        $template->set_attributes(array(
            'exercise_question' => $this->question,
            'exercise_hint'     => $this->hint,
            'answer_array'      => $this->answerArray,
            'student_array'     => $student_array,
            'evaluation_mode'   => $this->evaluationMode));

        return $template;
    }



    /**
     * Fills a WIKI template with all necessary values. If, additionally, a
     * solution is processed to it, it will also be set in.
     *
     * @param $solution (optional) the solution XML
     * @return the filled WIKI template object
     */
    function getWikiTemplate($solution_xml = null)
    {
        global $vipsTemplateFactory;

        // handle solution
        if (isset($solution_xml)) {
            $student_array = $this->getTagsFromXML($solution_xml, 'answer');
        }

        $template = $vipsTemplateFactory->open('exercises/wiki_sc_exercise');
        $template->set_attributes(array(
            'name'          => $this->name,
            'question'      => $this->question,
            'hint'          => $this->hint,
            'answer_array'  => $this->answerArray,
            'student_array' => $student_array
            ));

        return $template;
    }



    /**
     * Builds and fills a template which shows the corrected student solution.
     *
     * @param $solution_xml The XML of the student solution
     * @returns The template
     */
    function getCorrectedWikiTemplate($solution_xml)
    {
        global $vipsTemplateFactory;

        $student_array = $this->getTagsFromXML($solution_xml, 'answer');

        $template = $vipsTemplateFactory->open('exercises/wiki_sc_exercise_corrected');
        $template->set_attributes(array(
            'name'          => $this->name,
            'question'      => $this->question,
            'hint'          => $this->hint,
            'answer_array'  => $this->answerArray,
            'correct_array' => $this->correctArray,
            'student_array' => $student_array
            ));

        return $template;
    }



    /**
     * Constructs the <tt>answerArray</tt> from the REQUEST environment.
     *
     * @returns the answer array
     */
    function answerArrayFromInput()
    {
        // NOTE number_of_answers contains comma separated values here
        $num_answers = explode(',', vips_get_string_param('number_of_answers'));

        if ($num_answers[0] == '') {
            $num_answers = array();
        }

        $answer_array = array();

        for ($group = 0; $group < count($num_answers); ++$group) {
            $group_array = array();

            for ($alternative = 0; $alternative < $num_answers[$group]; ++$alternative) {
                $answer = vips_get_string_param('answer_'.$group.'_'.$alternative, true);

                if ($answer != '') {
                    $group_array[] = $answer;
                }
            }

            if (count($group_array)) {
                $answer_array[] = $group_array;
            }
        }

        return $answer_array;
    }



    /**
     * Constructs the <tt>correctArray</tt> from the REQUEST environment.
     *
     * @returns the correct array
     */
    function correctArrayFromInput()
    {
        // NOTE number_of_answers contains comma separated values here
        $num_answers = explode(',', vips_get_string_param('number_of_answers'));

        if ($num_answers[0] == '') {
            $num_answers = array();
        }

        $correct_array = array();

        for ($group = 0; $group < count($num_answers); ++$group) {
            $correct = vips_get_int_param('correct_'.$group);
            $shift = 0;

            for ($i = 0; $i < $num_answers[$group]; ++$i) {
                if (vips_get_string_param('answer_'.$group.'_'.$i, true) == '') {
                    if ($i == $correct) {
                        $correct_array[] = 0;
                    }
                    ++$shift;
                } else if ($i == $correct) {
                    $correct_array[] = $correct - $shift;
                }
            }

            if ($shift == $num_answers[$group]) {
                array_pop($correct_array);
            }
        }

        return $correct_array;
    }



    /**
     * Computes the number of answers.
     *
     * @returns An array with the number of answers in each group
     *          ([0] => 5, [1] => 3, [2] => 3, ...)
     */
    function computeNumberOfAnswers()
    {
        $number_of_answers = array();

        foreach ($this->answerArray as $group) {
            $number_of_answers[] = count($group);
        }

        return $number_of_answers;
    }


}  // end class

?>
