<?php
/*
 * tb_exercise.php - Vips plugin for Stud.IP
 * Copyright (c) 2003-2005  Erik Schmitt, Philipp Hgelmeyer
 * Copyright (c) 2005-2006  Christa Deiwiks
 * Copyright (c) 2006-2008  Elmar Ludwig, Martin Schrder
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 */

require_once 'exercises/exercise.php';

/**
 * XXX detailed description
 *
 * @author    XXX
 * @copyright XXX
 * @abstract
 */
class tb_exercise extends Exercise
{
    var $answerDefault,  // vorgegebener Text
        $answerDistance;  // hnlichkeitsberechnung: 0 = keine, 1 = Levenshtein

    // constructor
    function tb_exercise($xml = null)
    {
        parent::Exercise($xml);

        $this->type = 'tb_exercise';

        if (!isset($xml)) {  // construct exercise using request parameters
            $this->answerArray    = array(vips_get_string_param('answer_0', true));
            $this->answerDefault  = vips_get_string_param('answer_default', true);
            $this->answerDistance = vips_get_int_param('answer_distance');
        }
        else {  // construct exercise using XML
            $this->answerArray    = $this->getTagsFromXML($xml, 'Answer');
            $this->answerDefault  = reset($this->getTagsFromXML($xml, 'AnswerDefault'));
            $this->answerDistance = reset($this->getTagsFromXML($xml, 'AnswerDistance'));
        }
    }



    /**
     * Returns all variables which are peculiar for this exercise type.
     *
     * @returns An array with the keys 'answer_default' and 'answer_distance'
     */
    function getQuirks()
    {
        return array(
            'answer_default'  => $this->answerDefault,
            'answer_distance' => $this->answerDistance);
    }



    /**
     *    Evaluates a solution.
     *
     *    @abstract
     *    @access public
     *    @returns int (from 0 to 100 ) as an % value
     */

    function evaluate ($exercise, $solution)
    {
        $dom_xml = $this->getDomXML($exercise);
        $answers = $dom_xml->get_elements_by_tagname("Answer");
        $node = $answers[0];
        $musterLoesung  = utf8_decode($node->get_content());

        $answerDefault  = $this->getTagsFromXML($exercise, "AnswerDefault");
        $answerDefault  = $answerDefault[0];

        $answer_distance = $this->getTagsFromXML($exercise, "AnswerDistance");
        $answer_distance = $answer_distance[0];

        $studentSolution = $this->getTagsFromXML($solution, "answer");
        $studentSolution = $studentSolution[0];

        $similarity = 0; // between 0 and 1

        $answerDefault   = normalizeText($answerDefault);
        $studentSolution = normalizeText($studentSolution);
        $musterLoesung   = normalizeText($musterLoesung);

        if ($studentSolution == '' || $studentSolution == $answerDefault)
        {
            $m_return[1] = 0;       // no points
            $m_return[2] = 1;
            $m_return['percent'] = 0;
            $m_return['safe']    = 1;
            return $m_return;
        }

        if ($musterLoesung == $studentSolution)
        {
            $similarity = 1;
        }
        else if ($answer_distance == 1)
        {
            // similar_text, returns the number of matching characters
            // and not levenshtein, returns the number of insert, replace
            // and delete operations needed to transform str1 into str2.

            $divisor1 = max(strlen($answerDefault), strlen($studentSolution));
            $divisor2 = max(strlen($answerDefault), strlen($musterLoesung));
            $divisor3 = max(strlen($studentSolution), strlen($musterLoesung));

            $distance1 = similar_text($answerDefault, $studentSolution) / $divisor1;
            $distance2 = similar_text($answerDefault, $musterLoesung) / $divisor2;
            $distance3 = similar_text($studentSolution, $musterLoesung) / $divisor3;

            if ($answerDefault == $musterLoesung)
                $similarity = $distance3;
            else
                $similarity = min((1-$distance1) / (1-$distance2) * $distance3, 0.9);
        }
        else if ($answer_distance == 2)
        {
            // Soundex-values: four character string
            // [0] first letter of encoded expression
            // [1]-[3] digits, somehow representing the sound value

            $levenshtein = levenshtein(soundex($musterLoesung), soundex($studentSolution));

            if (soundex($musterLoesung) == soundex($studentSolution))
                $similarity = 0.8;
            else if ($levenshtein == 1)
                $similarity = 0.6;
            else if ($levenshtein == 2)
                $similarity = 0.4;
            else if ($levenshtein == 3)
                $similarity = 0.2;
            else  // $levenshtein == 4
                $similarity = 0;
        }

        $m_return[1] = $similarity * 100;       // percent
        $m_return[2] = $similarity == 1;
        $m_return['percent'] = $similarity * 100;
        $m_return['safe']    = $similarity == 1;
        return $m_return;
    }



    /**
     *    This function gens a Solution, that is stored to the database.
     *    solution_id.
     *
     *    @access public
     *    @returns XML contains the solution coded in an XML-string
     */
    function genSolution($exercise_id, $p_number_of_answers = null)
    {
        $xml = '<solution id="'.$exercise_id.'">';
        $xml .= '<answer id="0">';
        $xml .= '<body>'.htmlspecialchars(vips_get_string_param("answer_0", true)).'</body>';
        $xml .= '</answer>';
        $xml .= '</solution>';

        return vips_filter_xml_string($xml);
    }



    /**
     *    Generate the XML-input for the database for an exercise.
     *
     *    @abstract
     *    @access public
     *    @returns bool
     */

    function getExerciseConstruct()
    {
        $xmlop  = "<TestItem ID=\"" . $this->type . "\" Type=\"Text Box\" Language=\"" . $this->language . "\">";
        $xmlop .= "<Name>".htmlspecialchars($this->name)."</Name>";
        $xmlop .= "<Hint>".htmlspecialchars($this->hint)."</Hint>";
        $xmlop .= "<CommentField Visible=\"".$this->commentAllowed."\"></CommentField>";
        $xmlop .= "<AnswerDistance>".htmlspecialchars($this->answerDistance)."</AnswerDistance>";
        $xmlop .= "<AnswerDefault>".htmlspecialchars($this->answerDefault)."</AnswerDefault>";
        $xmlop .= "<Question>";
        $xmlop .= "<Paragraph>".htmlspecialchars($this->question)."</Paragraph>";
        $xmlop .= "</Question>";
        $xmlop .= "<Answer>".htmlspecialchars(reset($this->answerArray))."</Answer>";

        $xmlop .= "</TestItem>";

        return vips_filter_xml_string($xmlop);
    }

    /**
     * Export this exercise to plain text format.
     */
    function exportText ()
    {
        $result = parent::exportText('Offene Frage');

        $answer_text = $this->answerArray[0];

        if ($this->answerDistance == 1)
            $result .= "Abgleich: Levenshtein\n";

        if ($this->answerDefault != '')
        {
            $result .= "Vorgabe:\n";
            $result .= $this->answerDefault."\n";
            $result .= "\\Vorgabe\n";
        }

        if ($answer_text != '')
        {
            $result .= "Antwort:\n";
            $result .= $answer_text."\n";
            $result .= "\\Antwort\n";
        }

        return $result;
    }



    /**
     * Creates a template for editing a tb_exercise.
     *
     * @returns The template for editing this exercise
     */
    function getEditTemplate()
    {
        global $vipsTemplateFactory;

        $template = $vipsTemplateFactory->open('exercises/edit_tb_exercise');
        $template->set_attributes(array(
            'answer_array'   => $this->getAnswerArray(),
            'answer_default' => $this->answerDefault,
            'answer_distance' => $this->answerDistance));

        return $template;
    }



    /**
     * Creates a template for solving a tb_exercise.
     *
     * @returns The template
     */
    function getSolveTemplate($solution_xml = null)
    {
        global //$g_outlet,
               $vipsTemplateFactory;
        
//        $solver_user_id    = vips_get_string_param('solver_user_id');
//        $user_id           = isset($solver_user_id) ? $solver_user_id : $g_outlet->userID;
//        $exercise_position = vips_get_int_param('exercise_position');
//        $assignment_id     = vips_get_int_param('assignment_id');
//        $assignment_type   = vips_get_string_param('action') == 'sheets' ? 'vips_uebungsblatt' : 'vips_klausur';
//        $solution_xml      = getUserSolution($user_id, $exercise_position, $assignment_id, $assignment_type);
        if (isset($solution_xml)) {
            $student_array = $this->getTagsFromXML($solution_xml, 'answer');
        }

        $template = $vipsTemplateFactory->open('exercises/solve_tb_exercise');
        $template->set_attributes(array(
            'exercise_question' => $this->question,
            'exercise_hint'     => $this->exerciseHint,
            'student_array'     => $student_array,
            'answer_default'    => $this->answerDefault));

        return $template;
    }



    function getShowSolutionTemplate()
    {
    }
    
    function getShowCorrectionTemplate()
    {
    }
        
    function getPrintTemplate()
    {
    }



    /**
     * Fills a WIKI template with all necessary values. If, additionally, a
     * solution is processed to it, it will also be set in.
     *
     * @param $solution (optional) the solution XML
     * @return the filled WIKI template object
     */
    function getWikiTemplate($solution_xml = null)
    {
        global $vipsTemplateFactory;

        // handle solution
        if (isset($solution_xml)) {
            $student_array = $this->getTagsFromXML($solution_xml, 'answer');
        }

        $template = $vipsTemplateFactory->open('exercises/wiki_tb_exercise');
        $template->set_attributes(array(
            'name'           => $this->name,
            'question'       => $this->question,
            'hint'           => $this->hint,
            'answer_default' => $this->answerDefault,
            'answer_array'   => $this->answerArray,
            'student_array'  => $student_array
            ));

        return $template;
    }



    /**
     * Builds and fills a template which shows the corrected student solution.
     *
     * @param $solution_xml The XML of the student solution
     * @returns The template
     */
    function getCorrectedWikiTemplate($solution_xml)
    {
        global $vipsTemplateFactory;

        $student_array = $this->getTagsFromXML($solution_xml, 'answer');

        $template = $vipsTemplateFactory->open('exercises/wiki_tb_exercise_corrected');
        $template->set_attributes(array(
            'name'          => $this->name,
            'question'      => $this->question,
            'hint'          => $this->hint,
            'correct_array' => $this->answerArray,  // variable name confusion...
            'student_array' => $student_array
            ));

        return $template;
    }

}
?>
