<?php
/*
 *  $Id: ForeignKeyInfo.php,v 1.4 2005/03/01 13:56:43 micha Exp $
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many individuals
 * and is licensed under the LGPL. For more information please see
 * <http://creole.phpdb.org>.
 */

/**
 * Represents a foreign key.
 *
 * @author    Hans Lellelid <hans@xmpl.org>
 * @version   $Revision: 1.4 $
 * @package   creole.metadata
 */
class ForeignKeyInfo
{

  function NONE() { return ""; }            // No "ON [ DELETE | UPDATE]" behaviour specified.
//  function CASCADE() { return "NO ACTION"; }
  function CASCADE() { return "CASCADE"; }
  function RESTRICT() { return "RESTRICT"; }
//  function SETNULL() { return "SET DEFAULT"; }
  function SETNULL() { return "SET NULL"; }

  /**
  * @var string
  */
  var $name;
  /**
  * @var array
  */
  var $references = array();
  /**
  * Additional and optional vendor specific information.
  * @var array
  */
  var $vendorSpecificInfo = array();


  /**
  * ForeignKeyInfo constructor.
  *
  * @param string $name The name of the foreign key.
  * @param array $vendorInfo Vendor specific info array.
  */
  function ForeignKeyInfo($name, $vendorInfo = array())
  {
    $this->name = $name;
    $this->vendorSpecificInfo = $vendorInfo;
  }

  /**
  * Get foreign key name.
  *
  * @return string
  */
  function getName()
  {
    return $this->name;
  }

  /**
  * Adds a foreign-local mapping.
  *
  * @param ColumnInfo $local
  * @param ColumnInfo $foreign
  */
  function addReference(/*ColumnInfo*/ &$local, /*ColumnInfo*/ &$foreign, $onDelete = '', $onUpdate = '')
  {
    Creole::typeHint($local, 'ColumnInfo', 'ForeignKeyInfo', 'addReference', 1);
    Creole::typeHint($foreign, 'ColumnInfo', 'ForeignKeyInfo', 'addReference', 2);

    $this->references[] = array(&$local, &$foreign, $onDelete, $onUpdate);
  }

  /**
  * Gets the local-foreign column mapping.
  * @return array array( [0] => array([0] => local ColumnInfo object, [1] => foreign ColumnInfo object) )
  */
  function & getReferences()
  {
    return $this->references;
  }

  /**
  * Get vendor specific optional information for this primary key.
  *
  * @return array
  */
  function getVendorSpecificInfo()
  {
    return $this->vendorSpecificInfo;
  }

  /**
  * @return string
  */
  function toString()
  {
    return $this->name;
  }

}
