<?php
/*
 * vips_export.inc.php - Vips plugin for Stud.IP
 * Copyright (c) 2005-2006  Christa Deiwiks
 * Copyright (c) 2006-2008  Elmar Ludwig, Martin Schrder
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 */

require_once 'vips_assignments.inc.php';

/**
 *
 * Handles export of assignments, print preview etc (so far)
 *
 * @author: Christa Deiwiks <cdeiwiks@uos.de>
 */
function vips_export ($subaction)
{
    global $g_outlet, $g_assignment_table, $g_inAssignment_table,
           $g_solution_table, $g_solution_points_table;

    $type = vips_get_option_param('type');

    if (isset($type) && $type == 'exams') {
        $g_assignment_table      = 'vips_klausur';
        $g_inAssignment_table    = 'vips_inKlausur';
        $g_solution_table        = 'vips_K_Loesung';
        $g_solution_points_table = 'vips_K_Loesung_Punkte';
    } else {
        $g_assignment_table      = 'vips_uebungsblatt';
        $g_inAssignment_table    = 'vips_inUebungsblatt';
        $g_solution_table        = 'vips_T_Loesung';
        $g_solution_points_table = 'vips_T_Loesung_Punkte';
    }

    if ($g_outlet->status == 'lecturer' || $g_outlet->status == 'tutor') {
        if ($subaction == 'print_student_overview') {
            print_student_overview();
        } else if ($subaction == 'print_all_assignments') {
            print_all_assignments();
        } else if ($subaction == 'print_assignment') {
            print_assignment();
        } else {
            print_overview();
        }
    } else {
        if ($subaction == 'print_assignment') {
            print_assignment();
        } else {
            print_overview();
        }
    }
}

/**
 * NOTE: currently unused
 */
function main() {
        global $g_outlet, $vipsTemplateFactory;

        $t_main = $vipsTemplateFactory->open('export_main');

        $t_main->set_attributes(array(
                'picturepath' => $g_outlet->picturepath,
                'status'      => $g_outlet->status));

        echo $t_main->render();
}

/**
 * Lists all students of the course
 * => print solution of one/all student(s) for a given assignment
 *
 * get participants of course
 * list students with checkboxes to select one,two,all students
 * button:print assignment for selected students
 * new page for every student... pdf?
 */
function print_student_overview()
{
        global $vips_db, $g_outlet, $vipsTemplateFactory, $vipsPlugin;
        global $g_assignment_table, $g_inAssignment_table, $g_solution_table;

        $type   = vips_get_option_param("type");
        $mode   = vips_get_option_param("mode");
        $assignment     = vips_get_int_param("assignment");
        $print_correction_default      = vips_get_int_param("print_correction_default");
        $print_sample_solution_default = vips_get_int_param("print_sample_solution_default");

        $m_course_id = $g_outlet->courseID;

        // fetch name of exam/exercise sheet
        $query = "SELECT Name FROM $g_assignment_table WHERE ID = $assignment";
        $result = $vips_db->executeQuery($query);
        $result->next();
        $name = $result->get("Name");

        /////////////////////////////////////////////////////
        // Fetch all exercises that are in the assignment. //
        /////////////////////////////////////////////////////

        $m_all_exercises = array();

        $query = "SELECT DISTINCT
                        $g_inAssignment_table.`vips_aufgabe`
                        , `vips_aufgabe`.Name
                        , $g_inAssignment_table.`Position`
                FROM
                        $g_inAssignment_table
                        , `vips_aufgabe`
                WHERE
                        $g_inAssignment_table.$g_assignment_table = '$assignment'
                        AND vips_aufgabe.ID = $g_inAssignment_table.`vips_aufgabe`
                ORDER BY
                        $g_inAssignment_table.`Position`
                ";
        $result = $vips_db->executeQuery($query);

        while ($result->next())
        {
                $temp_ex = array
                        ( "id" => $result->get("vips_aufgabe")
                        , "name" => $result->get("Name")
                        , "position" => $result->get("Position")
                        );
                array_push($m_all_exercises, $temp_ex);
        }

        //////////////////////////////////////////////////////////////
        // Fetch all participants in this course from the database. //
        //////////////////////////////////////////////////////////////

        $m_participants = array();

        if ($g_outlet->status != 'student')
        {
                $query = "SELECT DISTINCT
                                seminar_user.user_id
                        FROM
                                seminar_user,
                                auth_user_md5
                        WHERE
                                Seminar_id = '$m_course_id' AND
                                seminar_user.user_id = auth_user_md5.user_id
                        ORDER BY Nachname, Vorname";
                $result = $vips_db->executeQuery($query);

                while ($result->next())
                {
                        $m_user_id = $result->get("user_id");

                        if (get_perm($m_course_id, $m_user_id) == "autor") { // skip tutors
                                $m_participants[] = array(
                                        "id"            => $m_user_id,
                                        "name"          => get_fullname($m_user_id),
                                        "username"      => get_username($m_user_id),
                                        "single_solver" => true);
                        }
                }
        }
        else
        {
                $m_participants[] = array(
                        "id"            => $g_outlet->userID,
                        "name"          => get_fullname($g_outlet->userID),
                        "username"      => get_username($g_outlet->userID),
                        "single_solver" => true);
        }

        $m_solvers = $m_participants;

        //////////////////////////////////////////////////////////////////////
        // Merge participants to groups if assignment is an exercise sheet. //
        //////////////////////////////////////////////////////////////////////
        # cdeiwiks: if no group, list as single participant

        if ($type == "sheets")
        {
                $m_solvers = array();
                $m_groups = array();

                foreach ($m_participants as $m_participant)
                {
                        $m_user_id = $m_participant["id"];
                        $fullname  = $m_participant["name"];
                        $username  = $m_participant["username"];
                        $m_group_id = '';

                        $query = "SELECT DISTINCT
                                        vips_gruppe.Gruppenid
                                        , vips_gruppe.Gruppenname
                                FROM
                                        vips_gruppe
                                        , vips_inGruppe
                                WHERE
                                        vips_gruppe.Kursid = '$m_course_id'
                                        AND vips_inGruppe.user_id = '$m_user_id'
                                        AND vips_inGruppe.group_id = vips_gruppe.Gruppenid
                                        AND vips_inGruppe.end IS NULL
                                ";   //cdeiwiks: ... IS NULL ist wichtig: bedeutet: Gruppe ist aktuell und Mitgliedschaft noch nicht beendet

                        $result = $vips_db->executeQuery($query);

                        if ($result->next())
                        {
                                $m_group_id = $result->get("Gruppenid");
                                $m_group_name = $result->get("Gruppenname");
                        }

                        if ($m_group_id == '')
                        {
                                $m_solvers[] = array(
                                        "id"            => $m_user_id,
                                        "name"          => $fullname,
                                        "username"      => $username,
                                        "single_solver" => true);
                        }
                        else
                        {
                            if (!is_array($m_groups[$m_group_name]))
                            {
                                    // $m_group_name as key is okay, as group name is unique
                                    $m_groups[$m_group_name] = array(
                                            "id"              => $m_group_id,
                                            "name"            => $m_group_name,
                                            "single_solver"   => false
                                            );
                            }

                            $m_groups[$m_group_name]['user'][] =
                                    array("name" => $fullname, "username" => $username);
                        }
                }

                ksort($m_groups);
                $m_solvers = array_merge($m_solvers, $m_groups);
        }

        //////////////////////////////////////////////////////////////////////
        // Fetch solution and points for each group/user for each exercise. //
        //////////////////////////////////////////////////////////////////////

        foreach ($m_solvers as $m_solver)
        {
                // look for solutions of this solver
                $m_solver_solutions = array();
                $query = "SELECT DISTINCT
                                $g_solution_table.`Aufgabenid` AS exerciseID
                        FROM
                                $g_solution_table
                        WHERE
                                Assignmentid = '$assignment'
                                AND ".($m_solver['single_solver'] ? 'Userid' : 'Gruppenid')." = '".$m_solver['id']."'
                                AND isLastSolution = 't'
                        ";
                $result = $vips_db->executeQuery($query);

                while ($result->next())
                {
                        $m_solver_solutions[$result->get("exerciseID")] = true;
                }

                // for all exercises in this assignment...
                foreach ($m_all_exercises as $m_exercise)
                {
                        if ($m_solver_solutions[$m_exercise['id']]) {
                                $students[] = $m_solver;
                                break;
                        }
                }
        }

        $t_main = $vipsTemplateFactory->open('export_student_overview');

        $t_main->set_attribute("type", $type);
        $t_main->set_attribute("assignment", $assignment);
        $t_main->set_attribute("name", $name);
        $t_main->set_attribute("students", $students);
        $t_main->set_attribute("mode", $mode);
        $t_main->set_attribute("print_correction_default", $print_correction_default);
        $t_main->set_attribute("print_sample_solution_default", $print_sample_solution_default);

        $t_main->set_attributes(array(
                'picturepath'    => $g_outlet->picturepath,
                'vips_imagepath' => $vipsPlugin->getPluginPath().'/images/',
                'status'         => $g_outlet->status));

        echo $t_main->render();
}

/**
 * print solution of one student for a given assignment and exercise id
 */
function print_student_solution ($exercise, $samplesolution = FALSE) {
        global $vips_db, $vipsTemplateFactory;

        // Database variables
        global $g_inAssignment_table, $g_solution_table, $g_solution_points_table,
                $g_assignment_table, $g_outlet;

        $type             = vips_get_option_param("type");
        $assignment       = vips_get_int_param("assignment");
        $user             = vips_get_option_param("user");
        $single           = vips_get_int_param("single");
        $print_correction = vips_get_int_param("print_correction");

        /*************************************************************************/
        /* Database: get exercise data for given exercise
        /*************************************************************************/

        $query = "SELECT vips_aufgabe.Aufgabe
                        , vips_aufgabe.URI
                        , vips_aufgabe.Name
                        , $g_inAssignment_table.Punkte
                        , $g_inAssignment_table.Position
                FROM vips_aufgabe
                        , $g_inAssignment_table
                WHERE vips_aufgabe.ID = '$exercise'
                        AND $g_inAssignment_table.vips_aufgabe = vips_aufgabe.ID
                        AND $g_inAssignment_table.$g_assignment_table = '$assignment'
                ";
        $result = $vips_db->executeQuery($query);
        if ($result->next())
        {
                $m_exercise_type       = $result->get("URI");
                $m_exercise_name       = $result->get("Name");
                $m_exercise_position   = $result->get("Position");
        }
        else
        {
            // FIXME generate error message here (no exercise data)
        }

        /*************************************************************************/
        /* Database: get solution data for given exercise and given group/user
        /*************************************************************************/

        if ($type == "sheets" && !$single)
        {
                $query = "SELECT Loesungsid, Loesung, visibleForStudent
                        FROM
                                ".$g_solution_table."
                        WHERE
                                Aufgabenid='$exercise' AND
                                Gruppenid = '$user' AND
                                Assignmentid='$assignment' AND
                                isLastSolution='t'";
        }
    else if ($type == "exams" || $single)
    {
            $query = "SELECT Loesungsid, Loesung, visibleForStudent
                                    FROM
                                            ".$g_solution_table."
                                    WHERE
                                            Aufgabenid='$exercise' AND
                                            Userid='$user' AND
                                            Assignmentid='$assignment' AND
                                            isLastSolution='t'";
    }

    $result = $vips_db->executeQuery($query);

        if ($result->next()) {
                $m_exercise_solution_id = $result->get("Loesungsid");
                $m_visible_for_student  = $result->get("visibleForStudent");
        }

        ###############################################################################
        # cdeiwiks vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
        #

        $query = "SELECT Punkte
                FROM $g_inAssignment_table
                WHERE $g_assignment_table = '$assignment'
                        AND vips_aufgabe = '$exercise'
                ";
        $result = $vips_db->executeQuery($query);
        if ($result->next())
        {
            $gesamtpunktzahl = $result->getFloat('Punkte');
        }

    #
    # cdeiwiks ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    ###################################################################################

        $query = "SELECT DISTINCT
                    Korrektur AS LecturerComment
                    , $g_solution_points_table.Punkte AS ReachedPoints
                    , $g_inAssignment_table.Punkte AS MaxPoints
            FROM
                    $g_inAssignment_table
                    , $g_solution_table
                    , $g_solution_points_table
            WHERE
                    $g_assignment_table = $assignment
                    AND vips_aufgabe = $exercise
                    AND Position = $m_exercise_position
                    AND Assignmentid = $assignment
                    AND Aufgabenid = $exercise
                    AND isLastSolution = 't'
                    AND idLoesung = Loesungsid
                    AND ".($type == "exams" || $single ? "Userid" : "Gruppenid")." = '$user'
                    ";
    $result = $vips_db->executeQuery($query);

    if ($result->next()) {
            $m_reached_points = $result->getFloat("ReachedPoints");
            $m_max_points = $result->getFloat("MaxPoints");
            $m_lecturer_comment = $result->get("LecturerComment");
    } else {
            // falls noch keine korrektur durchgefuehrt wurde
            $m_max_points = $gesamtpunktzahl;
    }

    // Unset $_POST variables so that rnd variables (which are not set for every
    // exercise) are deleted. If you do not do this, errors will occur as eg
    // $_POST["in_rnd_exercise"] won't ever become 0 again after a rnd exercise
    // in the assignment...
    unset($_POST['in_rnd_exercise']);
    unset($_POST['rnd_exercise_id']);
    unset($_POST['rnd_exercise_name']);
    unset($_POST['rnd_exercise_position']);

    if ($m_exercise_type == "rnd_exercise")
    {
        $_POST["in_rnd_exercise"]   = 1;
        $_POST["rnd_exercise_id"]   = $exercise;
        $_POST["rnd_exercise_name"] = $m_exercise_name;
        $_POST["rnd_exercise_position"] = $m_exercise_position;
    }

    $_POST["exercise_id"]       = $exercise;
    $_POST['submitted']         = $submitted;
    $_POST['exercise_position'] = $m_exercise_position;
    $_POST['print']             = 1;
    $_POST["samplesolution"]    = $samplesolution;
#   $_POST["assignment_type"]   = $type;  // needed in vips_assignments (?needed any more?)
    $_POST['assignment_id']     = $assignment;  // needed in vips_assignments
    $_POST["solver_id"]         = $user;  // needed in vips_assignments
    $_POST["single_solver"]     = $single;  // needed in vips_assignments
    $_POST["print_correction"]  = $print_correction;

    vips_assignments($type, "show_exercise");

    // template for showing lecturer's correction form
    $tmpl = $vipsTemplateFactory->open("solutions_edit_solution");

    $tmpl->set_attribute("t_reached_points", $m_reached_points);
    $tmpl->set_attribute("t_max_points", $m_max_points);
    $tmpl->set_attribute("t_lecturer_comment", $m_lecturer_comment);
    $tmpl->set_attribute("print", true);
    $tmpl->set_attribute("print_correction", $print_correction);

    setlocale(LC_NUMERIC, NULL);
    echo $tmpl->render();
    setlocale(LC_NUMERIC, 'C');

    return array('reached_points' => $m_reached_points, 'max_points' => $m_max_points);
}

/**
 * Creates html print view of a sheet/exam (new window) specified by id
 *
 * type = [sheets|exams]
 * mode = [blank|onlysample|student|samplesolution]   //blanko|blanko mit Musterloesung|studentenloesung ohne musterloesung|mit musterloesung]
 * assignment = sheet/exam id
 */
function print_assignment ($print_html_head = true) {
    global $vips_db, $g_outlet, $vipsTemplateFactory, $vipsPlugin;
    global $g_assignment_table, $g_inAssignment_table;

    $type                  = vips_get_option_param("type");
    $assignment            = vips_get_int_param("assignment");
    $user                  = vips_get_option_param("user");
    $single                = vips_get_int_param("single");
    $print_correction      = vips_get_int_param("print_correction");
    $print_sample_solution = vips_get_int_param("print_sample_solution");

    #######################
    # get assignment data #
    #######################

    $query = "SELECT * FROM $g_assignment_table WHERE ID = '$assignment'";
    $result = $vips_db->executeQuery($query);
    $result->next();
    $entry = $result->getRow();
    $assignment_name  = $entry['Name'];
    $assignment_start = $entry['Beginn'];
    $assignment_end   = $entry['Ende'];
    $printable        = $entry['Sichtbarkeit'];
    $show_corrections = $entry['korrekturenSichtbar'] == 't';
    $mc_auswertung    = $entry['mc_auswertung'];

    $query = "SELECT VeranstaltungsNummer FROM seminare WHERE Seminar_id = '$g_outlet->courseID'";
    $result = $vips_db->executeQuery($query);
    $result->next();
    $entry = $result->getRow();
    $course_number = $entry['VeranstaltungsNummer'];

    if ($g_outlet->status == 'student')
    {
        /* get group of student */
        $result = $vips_db->executeQuery("SELECT vips_inGruppe.group_id
            FROM vips_inGruppe, vips_gruppe
            WHERE vips_inGruppe.user_id = '$g_outlet->userID' AND
                vips_gruppe.Gruppenid = vips_inGruppe.group_id AND
                vips_gruppe.Kursid = '$g_outlet->courseID' AND
                vips_inGruppe.end IS NULL");
        if ($result->next())
        {
            $group_id = $result->get('group_id');
        }

        if (!$printable)
        {
            $error = _('Zugriff nur fr Dozenten erlaubt.');
        }
        else if ($user && ($single && $g_outlet->userID != $user || !$single && $group_id != $user))
        {
            $error = _('Zugriff nur auf eigene Klausuren erlaubt.');
        }
        else if (($print_sample_solution || $print_correction) && !$show_corrections)
        {
            $error = _('Zugriff erst nach Freischaltung durch den Dozenten mglich.');
        }

        if (isset($error))
        {
            $message = array('mode' => 'error', 'message' => $error);
            $_POST['message'] = $message;
            print_overview();
            return;
        }
    }

    $t_main_header = $vipsTemplateFactory->open('print_assignment_header');
    $t_main_header->set_attributes(array(
        'picturepath' => $g_outlet->picturepath,
        'status'      => $g_outlet->status));

    if ($print_html_head) {
        vips_output_begin('print');
    }

    ################
    # get semester #
    ################

    $result = $vips_db->executeQuery("SELECT semester_data.name FROM semester_data, seminare
        WHERE seminare.Seminar_id = '$g_outlet->courseID'
            AND semester_data.beginn = seminare.start_time");
    $result->next();
    $entry = $result->getRow();
    $semester = $entry['name'];    //z.B. WS 2005

    #################
    # get lecturers #
    #################

    $lecturers = array();

    $sql = "SELECT auth_user_md5.*
        FROM auth_user_md5, seminar_user
        WHERE seminar_user.seminar_id = '$g_outlet->courseID'
            AND auth_user_md5.user_id = seminar_user.user_id
            AND seminar_user.status = 'dozent'";
    $result = $vips_db->executeQuery($sql);

    while ($result->next()) {
        $lecturer = $result->getRow();
        $lecturers[] = $lecturer;
    }

    ####################
    # get participants #
    ####################

    if ($user)
    {
        if ($type == "sheets" && !$single)
        {
            // sort participants by last name, first name
            $query = "SELECT vips_inGruppe.user_id
                FROM vips_inGruppe,
                    auth_user_md5
                WHERE group_id = $user
                    AND vips_inGruppe.end IS NULL
                    AND vips_inGruppe.user_id = auth_user_md5.user_id
                ORDER BY Nachname, Vorname";
            $result = $vips_db->executeQuery($query);

            while ($result->next())
            {
                $user_id = $result->get('user_id');
                $students[] = get_fullname($user_id);
                $stud_ids[] = get_student_id($user_id);
            }
        }
        else
        {
            $students[] = get_fullname($user);
            $stud_ids[] = get_student_id($user);
        }
    }

    $t_main_header->set_attribute("course_name",$g_outlet->course);
    $t_main_header->set_attribute("course_number",$course_number);
    $t_main_header->set_attribute("lecturers",$lecturers);
    $t_main_header->set_attribute("semester",$semester);
    $t_main_header->set_attribute("students",$students);
    $t_main_header->set_attribute("stud_ids",$stud_ids);

    $t_main_header->set_attribute("assignment_name",$assignment_name);
    $t_main_header->set_attribute("assignment_start",$assignment_start);
    $t_main_header->set_attribute("assignment_end",$assignment_end);

    echo $t_main_header->render();

    if (!$user)  // without student solution
    {

        #####################
        # get all exercises #
        #####################

        $query = "SELECT vips_aufgabe.*,
                vips_aufgaben_typen.Typenname,
                $g_inAssignment_table.Position,
                $g_inAssignment_table.Punkte
            FROM vips_aufgabe,
                vips_aufgaben_typen,
                $g_inAssignment_table
            WHERE $g_assignment_table = '$assignment'
                AND vips_aufgabe = vips_aufgabe.ID
                AND vips_aufgaben_typen.URI = vips_aufgabe.URI
            ORDER BY $g_inAssignment_table.Position
            ";
        $result = $vips_db->executeQuery($query);
        $exercises = array();
        while ($result->next())
        {
            $res = $result->getRow();
            array_push($exercises, $res);
        }

        // process each exercise in the assignment
        while ($exercise = array_shift($exercises))
        {
            $exercise_id       = $exercise['ID'];
            $exercise_type     = $exercise['URI'];
            $exercise_title    = $exercise['Name'];
            $exercise_xml      = $exercise['Aufgabe'];
            $exercise_position = $exercise['Position'];
            $max_points        = (float) $exercise['Punkte'];

            // by default, show the given exercise body (which is one in number)
            $number_of_exercises_to_show = 1;

            // BUT: random exercise contains several subexercises -- show them all!
            if ($exercise_type == "rnd_exercise")
            {
                require_once("exercises/rnd_exercise.php");
                $rnd0_exercise = new rnd_exercise();
                // Attention: case sensitive!
                $subexercise_ids = $rnd0_exercise->getAttributesFromXML($exercise_xml, "SubExercise", "ID");
                // insert id of random exercise to the beginning of the array: this
                // will allow rnd-exercise to print something itself before all the
                // other subexercises are processed
                array_unshift($subexercise_ids, $exercise_id);
                $number_of_exercises_to_show = count($subexercise_ids);

                if ($number_of_exercises_to_show == 1)  // no subexercises!
                {
                    $rnd_exercise_empty = 1;
                }
                else
                {
                    $rnd_exercise_empty = 0;
                }
            }

            // echo exercise title
            setlocale(LC_NUMERIC, NULL);
            echo '<div style="height: 3em;"></div>';
            echo '<div style="page-break-inside: avoid">';
            echo '<table class="vips" style="width: 100%;">';
            echo '<tr><td>';
            echo '<b><u>'.$exercise_position.'. '.html_ready($exercise_title).'</u></b>';
            echo '</td><td style="text-align: right; white-space: nowrap;">';
            echo sprintf(ngettext('(%s Punkt)', '(%s Punkte)', $max_points), $max_points);
            echo "</td></tr></table>";
            setlocale(LC_NUMERIC, 'C');

            // now, show the exercise body/bodies
            for ($i = 0; $i < $number_of_exercises_to_show; $i++)
            {
                if ($exercise_type == "rnd_exercise")
                {
                    // backup of rnd exercise info
                    $in_rnd_exercise       = 1;
                    $rnd_exercise_id       = $exercise_id;
                    $rnd_exercise_title    = $exercise_title;
                    $rnd_exercise_xml      = $exercise_xml;
                    $rnd_exercise_position = $exercise_position;

                    $subexercise_id = array_shift($subexercise_ids);

                    // fetch info about the current SUBexercise
                    $query = "SELECT DISTINCT
                            ID
                            , URI
                            , Name
                            , Aufgabe
                        FROM
                            vips_aufgabe
                        WHERE
                            ID = '".$subexercise_id."'
                        ";
                    $result = $vips_db->executeQuery($query);
                    $result->next();

                    // store info about the subexercise
                    $exercise_id    = $result->get("ID");
                    $exercise_type  = $result->get("URI");
                    $exercise_title = $result->get("Name");
                    $exercise_xml   = $result->get("Aufgabe");
                }

                //////////////////////////// INFO ////////////////////////////
                // If we are in a random exercise, the $exercise_...        //
                // variables now contain information about the SUBexercise. //
                // Otherwise (default), these variables just describe the   //
                // current exercise.                                        //
                //////////////////////////////////////////////////////////////

                require_once("exercises/$exercise_type.php");
                $o_exercise = new $exercise_type($exercise_xml);

                $exercise_question = $o_exercise->getQuestion();
                $exercise_hint     = $o_exercise->getHint();
                $number_of_answers = $o_exercise->getNumberOfAnswers();
                $answer_array      = $o_exercise->getAnswerArray();
                $correct_array     = $o_exercise->getCorrectArray();
                $quirks            = $o_exercise->getQuirks();

                // TODO this needs to be moved into the pl_exercise class
                if ($exercise_type == 'pl_exercise') {
                    $answer = str_replace('m_l_', '', $answer_array[0]);
                    list($answer, $tests) = split('%T', $answer);
                    $answer_array[0] = $answer;
                }

                // for every exercise within the 'for' loop: echo exercise question
                echo "<p>";
                if ($i > 0)  // if there are any subexercises, show numbering before each one
                {
                    echo chr(ord($i)+48).')&nbsp;';  // changes "1" in "a", "2" in "b", ...
                }
                if ($exercise_type != 'cloze_exercise')
                {
                    echo formatReady($exercise_question);
                }
                echo '</p>';

                $t_main = $vipsTemplateFactory->open("exercises/show_$exercise_type");
                $t_main->set_attributes(array(
                    'exercise'           => $exercise,
                    'exercise_hint'      => $exercise_hint,
                    'answer_array'       => $answer_array,
                    'correct_array'      => $correct_array,
                    'mc_auswertung'      => $mc_auswertung,
                    'rnd_exercise_empty' => $rnd_exercise_empty,
                    'quirks'             => $quirks,
                    'print'              => true,
                    'samplesolution'     => $print_sample_solution,
                    'vips_imagepath'     => $vipsPlugin->getPluginPath().'/images/'));

                if ($exercise_type == 'zu_exercise') {
                    echo '<tr><td colspan="2">--- Keine Druckansicht mglich ---</td></tr><tr><td>&nbsp;</td></tr>';
                }
                else {
                    echo $t_main->render();
                }

                // restore rnd_exercise_data
                if ($in_rnd_exercise) {
                    $in_rnd_exercise = 0;
                    $exercise_id    = $rnd_exercise_id;
                    $exercise_type  = "rnd_exercise";
                    $exercise_title = $rnd_exercise_title;
                    $exercise_xml   = $rnd_exercise_xml;
                }
            }  // end for : for each SUBexercise (default: just one!)

            // echo exercise footer
            echo "</div>\n";
        }  // end while : for each exercise
        echo '</table>';
        echo '</div>';
    }  // end if : without student solution

    else  // if ($user) = with student solution
    {

        #####################
        # get all exercises #
        #####################

        $query = "SELECT vips_aufgabe.*,
                vips_aufgaben_typen.Typenname,
                $g_inAssignment_table.Position,
                $g_inAssignment_table.Punkte
            FROM vips_aufgabe,
                vips_aufgaben_typen,
                $g_inAssignment_table
            WHERE $g_assignment_table = '$assignment'
                AND vips_aufgabe = vips_aufgabe.ID
                AND vips_aufgaben_typen.URI = vips_aufgabe.URI
            ORDER BY $g_inAssignment_table.Position
            ";
        $result = $vips_db->executeQuery($query);

        while ($result->next()) {
            $exercise = $result->getRow();
            $exercises[] = $exercise['ID'];
        }
        $reached_points = 0;
        $max_points = 0;

        foreach ($exercises as $exercise) {
            $points_array = print_student_solution($exercise, $print_sample_solution);
            $reached_points += $points_array['reached_points'];
            $max_points     += $points_array['max_points'];
        }

        if ($print_correction)
        {
            // assure that no negative points are shown
            $reached_points = max(0, $reached_points);

            $tmpl = $vipsTemplateFactory->open('export_print_footer');
            $tmpl->set_attribute('reached_points', $reached_points);
            $tmpl->set_attribute('max_points', $max_points);
            setlocale(LC_NUMERIC, NULL);
            echo $tmpl->render();
            setlocale(LC_NUMERIC, 'C');
        }
    }  // end if : with student solution

    if ($print_html_head) {
        vips_output_end();
    }
}

/**
 * Creates html print view of a sheet/exam (new window) specified by id
 * This function just calls print_assignment() for each user or group in turn.
 *
 * type = [sheets|exams]
 * mode = [student|samplesolution]   //[ohne musterloesung|mit musterloesung]
 * assignment = sheet/exam id
 */
function print_all_assignments ()
{
    $user = vips_get_option_array_param("user");
    $single = vips_get_int_array_param("single");

    vips_output_begin('print');

    // when printing for all students, $user is an array
    for ($i = 0; $i < count($user); ++$i)
    {
        $_POST['user'] = $user[$i];
        $_POST['single'] = $single[$i];

        print_assignment(false);
        echo '<p style="page-break-after: always;">';
    }

    vips_output_end();
}

/**
 * Displays overview of all assignments.
 */
function print_overview() {
        global $vips_db, $g_outlet, $vipsTemplateFactory, $vipsPlugin;

        $t_main = $vipsTemplateFactory->open('export_print_overview');
        $sheets = array();
        $exams = array();

        $t_main->set_attribute('message', vips_get_string_array_param('message'));

        $query = "SELECT * FROM vips_uebungsblatt
                  WHERE Kurs = '$g_outlet->courseID'
                  ORDER BY Beginn, Position";

        $result = $vips_db->executeQuery($query);

        while ($result->next()) {
                $entry = $result->getRow();

                if ($g_outlet->status != 'student' || $entry['Sichtbarkeit']) {
                        $sheets[] = $entry;
                }
        }

        $query = "SELECT * FROM vips_klausur
                  WHERE Kurs='$g_outlet->courseID'
                  ORDER BY Beginn, Position";

        $result = $vips_db->executeQuery($query);

        while ($result->next()) {
                $entry = $result->getRow();

                if ($g_outlet->status != 'student' || $entry['Sichtbarkeit']) {
                        $exams[] = $entry;
                }
        }

        /* get group of student */
        $result = $vips_db->executeQuery("SELECT vips_inGruppe.group_id
            FROM vips_inGruppe, vips_gruppe
            WHERE vips_inGruppe.user_id = '$g_outlet->userID' AND
                vips_gruppe.Gruppenid = vips_inGruppe.group_id AND
                vips_gruppe.Kursid = '$g_outlet->courseID' AND
                vips_inGruppe.end IS NULL");

        if ($result->next())
        {
                $group_id = $result->get('group_id');
        }

        $t_main->set_attribute('sheets', $sheets);
        $t_main->set_attribute('exams', $exams);

        $t_main->set_attributes(array(
                'user'           => $g_outlet->userID,
                'group_id'       => $group_id,
                'picturepath'    => $g_outlet->picturepath,
                'vips_imagepath' => $vipsPlugin->getPluginPath().'/images/',
                'status'         => $g_outlet->status));

        echo $t_main->render();
}

?>
